#include <vector>
#include <queue>
#include <stack>
#include <list>
#include <map>
#include <set>
#include <string>
#include <sstream>
#include <iostream>
#include <cmath>
#include <ctime>
#include <cstring>
#include <algorithm>
#include <windows.h>

#pragma warning( disable: 4996 )

using namespace std;

typedef unsigned long long ull;

template<typename T> inline T sqr(T a) { return a*a; }
template<typename T> inline int size(const T& c) { return (int)c.size(); }

#define zero(a) memset( &a, 0, sizeof(a) )

//#define printf
//#define DBG
//#define VERIFY
#define ERR         -1

#define MAXN        300

LARGE_INTEGER Start, Now, Diff, Freq;

int N, M;
int D[ MAXN ][ MAXN ], A[ MAXN ][ MAXN ], X[ MAXN ][ MAXN ], W[ MAXN ][ MAXN ];
int P[ MAXN ], C[ MAXN ], R[ MAXN ], U[ MAXN ], V[ MAXN ];
ull S[ MAXN ];
ull SS[ MAXN ][ MAXN ];
pair< int, ull > Q[ MAXN+1 ];
vector< int > T[ MAXN ], G[ MAXN ];
vector< pair< ull, int > > F;
set< int > E[ MAXN ];
set< int > L;
ull good, best = (ull) -1;
int start_idx, edge_idx;


void dump_time() {

    long long time_in_ms;

    QueryPerformanceCounter( &Now );
    Diff.QuadPart = Now.QuadPart - Start.QuadPart;
    time_in_ms = ( Diff.QuadPart * 1000 ) / Freq.QuadPart;;
    printf(" %llu\n", time_in_ms);
}


long long get_time() {

    long long time_in_ms;

    QueryPerformanceCounter( &Now );
    Diff.QuadPart = Now.QuadPart - Start.QuadPart;
    time_in_ms = ( Diff.QuadPart * 1000 ) / Freq.QuadPart;

    return time_in_ms;
}


int read_input() {

    FILE *fin = fopen( "graph.in", "r" );
    int i, j, x, y, d;

    if ( fin == NULL ) {

        printf( "Failed to open input file!\n" );
        return ERR;
    }

    fscanf( fin, "%d %d", &N, &M );

    for ( i = 0; i < M; i++ ) {

        fscanf( fin, "%d %d %d", &x, &y, &d );
        D[ x-1 ][ y-1 ] = D[ y-1 ][ x-1 ] = d;
        X[ x-1 ][ y-1 ] = X[ y-1 ][ x-1 ] = i+1;
        G[ x-1 ].push_back( y-1 );
        G[ y-1 ].push_back( x-1 );
    }

    for ( i = 0; i < N; i++ )
        for ( j = 0; j < N; j++ )
            fscanf( fin, "%d", &A[ i ][ j ] );

    fclose( fin );

    return 0;
}


int write_output() {

    FILE *fout = fopen( "graph.out", "w+" );
    int i;

    //printf("\n\nfinal -> best = %llu. improved by = %llu. time = ", best, good - best);
    //dump_time();

    if ( fout == NULL ) {

        printf( "Failed to create output file!\n" );
        return ERR;
    }

    for ( i = 0; i < N-1; i++ )
        fprintf( fout, "%d\n", R[ i ] );

    fclose( fout );

    return 0;
}


void relax_tree() {

    int i, j, k;

    for ( k = 0; k < N; k++ )
        for ( i = 0; i < N; i++ )
            for ( j = 0; j < N; j++ )
                if ( ( i != k ) && ( k != j ) && ( i != j ) &&
                    ( D[ i ][ k ] ) && ( D[ k ][ j ] ) && ( D[ i ][ j ] ) &&
                    ( D[ i ][ k ] + D[ k ][ j ] <= D[ i ][ j ] ) ) {

                        D[ i ][ j ] = D[ i ][ k ] + D[ k ][ j ];
                        D[ j ][ i ] = D[ i ][ k ] + D[ k ][ j ];
                        W[ i ][ j ] = -1;
                        W[ j ][ i ] = -1;
                }

    for ( i = 0; i < N; i++ )
        for ( j = 0; j < N; j++ )
            if ( W[ i ][ j ] < 0 ) {

                //printf("remove edge (%d,%d)\n", i, j);
                D[ i ][ j ] = 0;
            }

    for ( i = 0; i < N; i++ ) {

        G[ i ].clear();

        for ( j = 0; j < N; j++ )
            if ( D[ i ][ j ] > 0 ) {

                G[ i ].push_back( j );
            }
    }
}


void clear_tree( ) {

    int i;

    for ( i = 0; i < N; i++ ) {

        T[i].clear();
        E[i].clear();
    }

    L.clear();
}


void generate_set_tree( ) {

    int i, j;

    for ( i = 0; i < N; i++ ) {

        E[ i ].clear();

        for ( j = 0; j < size( T[ i ] ); j++ )
            E[ i ].insert( T[ i ][ j ] );

        if ( size( E[ i ] ) == 1 )
            L.insert( i );
    }
}


void dump_tree( ) {

    int i, j;
    set<int>::iterator it;

    for ( i = 0; i < N; i++ ) {

        printf("v[%d] -> ", i);
        for ( it = E[ i ].begin(); it != E[ i ].end(); it++ ) printf("%d ", *it);
        printf("\n");
    }

    printf("\n");

    for ( i = 0; i < N; i++ ) {

        for ( j = 0; j < N; j++ )
            printf("%llu ", SS[i][j]);

        printf("\n");
    }

    printf("\n");
}


ull calculate_dist_from_vertex( int s ) {

    int head, tail;
    int i, v, w;
    ull d;

    zero( U );

    head = tail = 0;

    S[ s ] = 0;
    U[ s ] = 1;

    Q[ tail ].first = s;
    Q[ tail ].second = 0;
    tail++;

    while ( head < tail ) {

        v = Q[ head ].first;
        d = Q[ head ].second;
        head++;

        for ( i = 0; i < size( T[ v ] ); i++ )
            if ( !U[ T[ v ][ i ] ] ) {

                w = T[ v ][ i ];

                SS[ s ][ w ] = d + D[ v ][ w ];
                S[ s ] += SS[ s ][ w ] * (ull) A[ s ][ w ];

                Q[ tail ].first = w;
                Q[ tail ].second = SS[ s ][ w ];
                tail++;

                U[ w ] = 1;
            }
    }

    return S[ s ];
}


void save_solution( int v, bool save_set_tree ) {

    if ( save_set_tree ) {

        set< int >::iterator it;

        U[ v ] = 1;

        for ( it = E[ v ].begin(); it != E[ v ].end(); it++ )
            if ( !U[ *it ] ) {

                R[ edge_idx++ ] = X[ v ][ *it ];

                save_solution( *it, save_set_tree );
            }

        U[ v ] = 0;

    } else {

        int i;

        U[ v ] = 1;

        for ( i = 0; i < size( T[ v ] ); i++ )
            if ( U[ T[ v ][ i ] ] == 0 ) {

                R[ edge_idx++ ] = X[ v ][ T[ v ][ i ] ];

                save_solution( T[ v ][ i ], save_set_tree );
            }

        U[ v ] = 0;
    }
}


ull evaluate_tree( bool precalculate ) {

    int i;
    set<int>::iterator it;
    long long time_in_ms;
    ull tt = 0;

    time_in_ms = get_time();

    if ( time_in_ms > 1900 ) {

        write_output();
        exit(0);
    }

    for ( i = 0; i < N; i++ ) {

        if ( precalculate ) {

            calculate_dist_from_vertex( i );
        }

        tt += S[i];
    }

    if ( ( precalculate ) && ( size( F ) < N ) ) {

        F.push_back( make_pair( tt, start_idx ) );
    }

    if ( tt < best ) {

        zero( U );

        edge_idx = 0;
        best = tt;

        save_solution( 0, !precalculate );

        if ( edge_idx != N-1 ) return -777;
    }

    return tt;
}


int generate_tree_from_sequence( ) {

    priority_queue< int, vector<int>, greater<int> > pq;
    int i, v;

    zero( C );

    for ( i = 0; i < N-2; i++ )
        C[ P[ i ] ]++;

    for ( i = 0; i < N; i++ )
        if ( C[ i ] == 0 ) pq.push( i );

    for ( i = 0; i < N-2; i++ ) {

        v = pq.top();
        pq.pop();

        if ( D[ v ][ P[ i ] ] == 0 ) return ERR;

        T[ v ].push_back( P[ i ] );
        T[ P[ i ] ].push_back( v );

        if ( --C[ P[ i ] ] == 0 ) pq.push( P[ i ] );
    }

    v = pq.top();
    pq.pop();

    if ( D[ v ][ N-1 ] == 0 ) return ERR;

    T[ v ].push_back( N-1 );
    T[ N-1 ].push_back( v );

    return 0;
}


int transform_tree( int target, int from, int to ) {

    int i;
    bool from_must_inserted_in_leafs, to_must_be_removed_from_leafs;

    from_must_inserted_in_leafs = ( E[from].size() <= 2 );
    to_must_be_removed_from_leafs = ( E[to].size() == 1 );

    E[ from ].erase( target );
    E[ target ].erase( from );

    E[ to ].insert( target );
    E[ target ].insert( to );

    SS[ target ][ to ] = D[ target ][ to ];
    SS[ to ][ target ] = D[ to ][ target ];

    SS[ target ][ from ] = D[ target ][ to ] + SS[ to ][ from ];
    SS[ from ][ target ] =  SS[ from ][ to ] + D[ to ][ target ];

    for ( i = 0; i < N; i++ )
        if ( ( i != target ) && ( i != from ) && ( i != to ) ) {

            S[ i ] = S[ i ] - ( SS[ i ][ target ] - SS[ i ][ to ] - D[ to ][ target ] ) * (ull) A[ i ][ target ];

            SS[ target ][ i ] = D[ target ][ to ] + SS[ to ][ i ];
            SS[ i ][ target ] =  SS[ i ][ to ] + D[ to ][ target ];
        }

    S[ target ] = S[ from ] = S[ to ] = 0;

    for ( i = 0; i < N; i++ ) {

        S[ target ] += SS[ target ][ i ] * (ull) A[ target ][ i ];
        S[ from ] += SS[ from ][ i ] * (ull) A[ from ][ i ];
        S[ to ] += SS[ to ][ i ] * (ull) A[ to ][ i ];
    }

    if ( from_must_inserted_in_leafs ) L.insert( from );
    if ( to_must_be_removed_from_leafs ) L.erase( to );

    return 0;
}


void morph_tree( int x, int y, int z ) {

    int i, j;
    int target, from, to;
    int mfrom = -1, mto = -1, mtarget = -1;
    ull tt, ttx, mttx = (ull)-1;
    vector<int> LV( L.begin(), L.end() );

    tt = evaluate_tree( false );

    for ( i = 0; i < size( LV ); i++ ) {

        target = LV[ i ];

        if ( size( E[target] ) != 1 ) {

            return;
        }

        from = *( E[ target ].begin() );

        for ( j = 0; j < size( G[ target ] ); j++ ) {

            to = G[ target ][ j ];

            if ( ( to != from ) && ( to != target ) && ( D[to][target] != 0 ) && !( x == target && y == from && z == to) ) {

                transform_tree( target, from, to );
                ttx = evaluate_tree( false );

                if ( ttx < mttx ) {

                    mttx = ttx;
                    mtarget = target;
                    mfrom = from;
                    mto = to;
                }

                transform_tree( target, to, from );
            }
        }
    }

    if ( mttx < tt ) {

        transform_tree( mtarget, mfrom, mto );
        ttx = evaluate_tree( false );
        morph_tree( mtarget, mto, mfrom );
        transform_tree( mtarget, mto, mfrom );
    }

}


void generate_all_trees( int idx ) {

    int i, res;

    if ( idx == N-2 ) {

        res = generate_tree_from_sequence();
        if ( res != ERR ) evaluate_tree( true );
        clear_tree();

        return;
    }

    for ( i = 0; i < N; i++ ) {

        P[ idx ] = i;
        generate_all_trees( idx+1 );
    }
}


int generate_dijkstra_tree( int s ) {

    int i;

    start_idx = s;

    zero( S );

    priority_queue< pair< ull,int >, vector< pair< ull, int > >, greater< pair< ull, int > > > pq;
    int vx, vy;
    ull dx;

    for ( i = 0; i < N; i++ ) {

        S[ i ] = (ull) -1;
        P[ i ] = s;
    }

    S[ s ] = 0;
    P[ s ] = -1;

    pq.push( make_pair( 0, s ) );

    while ( !pq.empty() ) {

        pair< ull, int > pi = pq.top();
        pq.pop();

        dx = pi.first;
        vx = pi.second;

        if ( dx <= S[ vx ] ) {

            for ( i = 0; i < size( G[ vx ] ); i++ ) {

                vy = G[ vx ][ i ];

                if ( ( vy != s ) && ( D[ vx ][ vy ] ) && ( S[ vx ] + D[ vx ][ vy ] < S[ vy ] ) ) {

                    S[ vy ] = S[ vx ] + D[ vx ][ vy ];
                    pq.push( make_pair( S[ vy ], vy ) );
                    P[ vy ] = vx;
                }
            }
        }
    }

    for ( i = 0; i < N; i++ ) {

        if ( P[ i ] != -1 ) {

            T[ i ].push_back( P[ i ] );
            T[ P[ i ] ].push_back( i );
        }
    }

    return 0;
}


int main() {

    int i, res;

    srand( (int)time(NULL) );

    if ( !QueryPerformanceFrequency( &Freq ) ) {

        printf("QueryPerformanceFrequency() failed! Error = %d\n", GetLastError());
        return ERR;
    }

    if ( !QueryPerformanceCounter( &Start ) ) {

        printf("QueryPerformanceCounter() failed! Error = %d\n", GetLastError());
        return ERR;
    }

    if ( read_input() == ERR ) {

        printf( "read_input() failed!\n" );
        return ERR;
    }

/*
    int j;

    N = 300;

    for ( i = 0; i < N; i++ )
        for ( j = i+1; j < N; j++ ) {

            D[i][j] = D[j][i] = rand() % 1000 + 1;
            if ( (i+j)%70 == 0 ) A[i][j] = A[j][i] = rand() % 1000000000 + 1;
            else A[i][j] = A[j][i] = rand() % 4 + 1;
            //D[i][j] = D[j][i] = A[i][j] = A[j][i] = 1;
            //A[i][j] = A[j][i] = 1;
            G[i].push_back( j );
            G[j].push_back( i );
            M++;
        }

    int idx = 1;

    for ( i = 0; i < N; i++ )
        for ( j = i+1; j < N; j++ )
            X[i][j] = X[j][i] = idx++;
*/
    relax_tree();

    if ( N == 1 ) {

        // do nothing
    } else if ( N == 2 ) {

        // single edge
        R[0] = 1;
    } else if ( N < 9 ) {

        generate_all_trees( 0 );
    } else {

        //printf("pre-processing -> time = ");
        //dump_time();

        for ( i = 0; i < N; i++ ) {

            res = generate_dijkstra_tree( i );
            if ( res != ERR ) evaluate_tree( true );
            clear_tree();

            if ( get_time() > 1700 ) break;
        }

        printf("dijkstra -> best = %llu. time = ", best);
        dump_time();

        good = best;

        sort( F.begin(), F.end() );

        for ( i = 0; i < size(F); i++ ) {

            //printf("morph_tree [%d] -> old_best = %llu. ", F[i].second, best);

            res = generate_dijkstra_tree( F[i].second );
            if ( res != ERR ) evaluate_tree( true );
            generate_set_tree();
            morph_tree( -1, -1, -1 );
            clear_tree();

            //printf("new_best = %llu. time = ", best);
            //dump_time();
        }
    }

    if ( write_output() == ERR ) {

        printf( "write_output() failed!\n" );
        return ERR;
    }

    return 0;
}
